# ----------------------------------------------------------------------------
# Copyright (c) 2017-2018 FIRST. All Rights Reserved.
# Open Source Software - may be modified and shared by FRC teams. The code
# must be accompanied by the FIRST BSD license file in the root directory of
# the project.
# ----------------------------------------------------------------------------

import rev
import wpilib


class Robot(wpilib.TimedRobot):
    # This sample program displays the position and velocity of the integrated
    # encoder onto the SmartDashboard.
    #
    # Position is displayed in revolutions (of the motor's axle) and velocity
    # is displayed in revolutions per minute (RPM)
    #
    # Optionally, if you call the setPositionConversionFactor() method on the
    # encoder and give it a measurement of how far one revolution is, the
    # getVelocity() and getPosition() methods return a scaled output in the
    # units of your choice.
    def robotInit(self):
        # Instantiate SPARK MAX object
        self.motor = rev.CANSparkMax(1, rev.MotorType.kBrushless)

        self.motor.restoreFactoryDefaults()
        self.encoder = self.motor.getEncoder()

        self.joystick = wpilib.Joystick(0)

    def teleopPeriodic(self):
        # Set motor output to the joystick's Y-axis
        self.motor.set(self.joystick.getY())

        # Encoder position is read from a CANEncoder object by calling the
        # getPosition() method.
        #
        # getPosition() returns the position of the encoder in units of
        # revolutions (unless overridden)
        wpilib.SmartDashboard.putNumber("Encoder Position", self.encoder.getPosition())

        # Encoder velocity is read from a CANEncoder object by calling the
        # getVelocity() method.
        #
        # getVelocity() returns the position of the encoder in units of
        # revolutions (unless overridden)
        wpilib.SmartDashboard.putNumber("Encoder Velocity", self.encoder.getVelocity())


if __name__ == "__main__":
    wpilib.run(Robot)



# ----------------------------------------------------------------------------
# Copyright (c) 2017-2018 FIRST. All Rights Reserved.
# Open Source Software - may be modified and shared by FRC teams. The code
# must be accompanied by the FIRST BSD license file in the root directory of
# the project.
# ----------------------------------------------------------------------------

import rev
import wpilib


class Robot(wpilib.TimedRobot):
    def robotInit(self):
        # Create motors
        self.motor = rev.CANSparkMax(1, rev.MotorType.kBrushless)

        # You must call getPIDController() on an existing CANSparkMax or
        # SparkMax object to fully use PID functionality
        self.pidController = self.motor.getPIDController()

        # Instantiate built-in encoder to display position
        self.encoder = self.motor.getEncoder()

        self.joystick = wpilib.Joystick(0)

        # PID Coefficents and Controller Output Range
        self.kP = 0.1
        self.kI = 1e-4
        self.kD = 0
        self.kIz = 0
        self.kFF = 0
        self.kMinOutput = -1
        self.kMaxOutput = 1

        # Motor max RPM
        self.maxRPM = 5700

        # The restoreFactoryDefaults() method can be used to reset the
        # configuration parameters in the SPARK MAX to their factory default
        # state. If no argument is passed, these parameters will not persist
        # between power cycles
        self.motor.restoreFactoryDefaults()

        # Set PID Constants
        self.pidController.setP(self.kP)
        self.pidController.setI(self.kI)
        self.pidController.setD(self.kD)
        self.pidController.setIZone(self.kIz)
        self.pidController.setFF(self.kFF)
        self.pidController.setOutputRange(self.kMinOutput, self.kMaxOutput)

        # Push PID Coefficients to SmartDashboard
        wpilib.SmartDashboard.putNumber("P Gain", self.kP)
        wpilib.SmartDashboard.putNumber("I Gain", self.kI)
        wpilib.SmartDashboard.putNumber("D Gain", self.kD)
        wpilib.SmartDashboard.putNumber("I Zone", self.kIz)
        wpilib.SmartDashboard.putNumber("Feed Forward", self.kFF)
        wpilib.SmartDashboard.putNumber("Min Output", self.kMinOutput)
        wpilib.SmartDashboard.putNumber("Max Output", self.kMaxOutput)

    def teleopPeriodic(self):
        # Read data from SmartDashboard
        p = wpilib.SmartDashboard.getNumber("P Gain", 0)
        i = wpilib.SmartDashboard.getNumber("I Gain", 0)
        d = wpilib.SmartDashboard.getNumber("D Gain", 0)
        iz = wpilib.SmartDashboard.getNumber("I Zone", 0)
        ff = wpilib.SmartDashboard.getNumber("Feed Forward", 0)
        min_out = wpilib.SmartDashboard.getNumber("Min Output", 0)
        max_out = wpilib.SmartDashboard.getNumber("Max Output", 0)

        # Update PIDController datapoints with the latest from SmartDashboard
        if p != self.kP:
            self.pidController.setP(p)
            self.kP = p
        if i != self.kI:
            self.pidController.setI(i)
            self.kI = i
        if d != self.kD:
            self.pidController.setD(d)
            self.kD = d
        if iz != self.kIz:
            self.pidController.setIZone(iz)
            self.kIz = iz
        if ff != self.kFF:
            self.pidController.setFF(ff)
            self.kFF = ff
        if (min_out != self.kMinOutput) or (max_out != self.kMaxOutput):
            self.pidController.setOutputRange(min_out, max_out)
            self.kMinOutput = min_out
            self.kMaxOutput = max_out

        setpoint = self.maxRPM * self.joystick.getY()

        # PIDController objects are commanded to a set point using the
        # setReference() method.
        #
        # The first parameter is the value of the set point, whose units vary
        # depending on the control type set in the second parameter.
        #
        # The second parameter is the control type can be set to one of four
        # parameters:
        # rev.ControlType.kDutyCycle
        # rev.ControlType.kPosition
        # rev.ControlType.kVelocity
        # rev.ControlType.kVoltage
        #
        # For more information on what these types are, refer to the Spark Max
        # documentation.
        self.pidController.setReference(setpoint, rev.ControlType.kVelocity)

        # Push Setpoint and the motor's current velocity to SmartDashboard.
        wpilib.SmartDashboard.putNumber("SetPoint", setpoint)
        wpilib.SmartDashboard.putNumber("ProcessVariable", self.encoder.getVelocity())


if __name__ == "__main__":
    wpilib.run(Robot)



# from unittest.mock import patch

# import pytest


# @pytest.fixture(scope="function")
# def _module_patch(request):
#     """This patch forces hal to reload each time we do this"""

#     # .. this seems inefficient

#     m = patch.dict("sys.modules", {})
#     m.start()
#     try:
#         yield
#     finally:
#         m.stop()


# @pytest.fixture(scope="function")
# def hal(_module_patch):
#     """Simulated hal module"""
#     import hal

#     return hal


# @pytest.fixture(scope="function")
# def hal_data(hal):
#     """Simulation data for HAL"""
#     import hal_impl.functions
#     import hal_impl.data

#     hal_impl.functions.reset_hal()
#     return hal_impl.data.hal_data


# @pytest.fixture(scope="function")
# def rev(hal_data):
#     import rev

#     return rev



from .version import version as __version__

from . import _init_rev

# autogenerated by 'robotpy-build create-imports rev rev._rev'
from ._rev import (
    CANAnalog,
    CANDigitalInput,
    CANEncoder,
    CANError,
    CANPIDController,
    CANSensor,
    CANSparkMax,
    CANSparkMaxLowLevel,
    ControlType,
    SparkMax,
)

__all__ = [
    "CANAnalog",
    "CANDigitalInput",
    "CANEncoder",
    "CANError",
    "CANPIDController",
    "CANSensor",
    "CANSparkMax",
    "CANSparkMaxLowLevel",
    "ControlType",
    "SparkMax",
]

# backwards compat
MotorType = CANSparkMaxLowLevel.MotorType
IdleMode = CANSparkMax.IdleMode
LimitSwitch = CANDigitalInput.LimitSwitch
LimitSwitchPolarity = CANDigitalInput.LimitSwitchPolarity

__all__ += [
    "MotorType",
    "IdleMode",
    "LimitSwitch",
    "LimitSwitchPolarity",
]



# ----------------------------------------------------------------------------
# Copyright (c) 2017-2018 FIRST. All Rights Reserved.
# Open Source Software - may be modified and shared by FRC teams. The code
# must be accompanied by the FIRST BSD license file in the root directory of
# the project.
# ----------------------------------------------------------------------------

import rev
import wpilib

# Before Running:
# Open Shuffleboard, select File->Load Layout and select the
# shuffleboard.json that is in the root directory of this example


class Robot(wpilib.TimedRobot):
    def robotInit(self):
        # Create motors
        self.motor = rev.CANSparkMax(1, rev.MotorType.kBrushless)

        # You must call getPIDController() on an existing CANSparkMax or
        # SparkMax object to fully use PID functionality
        self.pidController = self.motor.getPIDController()

        # Instantiate built-in encoder to display position
        self.encoder = self.motor.getEncoder()

        # PID Coefficents and Controller Output Range
        self.kP = 0.1
        self.kI = 1e-4
        self.kD = 0
        self.kIz = 0
        self.kFF = 0
        self.kMinOutput = -1
        self.kMaxOutput = 1

        # The restoreFactoryDefaults() method can be used to reset the
        # configuration parameters in the SPARK MAX to their factory default
        # state. If no argument is passed, these parameters will not persist
        # between power cycles
        self.motor.restoreFactoryDefaults()

        # Set PID Constants
        self.pidController.setP(self.kP)
        self.pidController.setI(self.kI)
        self.pidController.setD(self.kD)
        self.pidController.setIZone(self.kIz)
        self.pidController.setFF(self.kFF)
        self.pidController.setOutputRange(self.kMinOutput, self.kMaxOutput)

        # Push PID Coefficients to SmartDashboard
        wpilib.SmartDashboard.putNumber("P Gain", self.kP)
        wpilib.SmartDashboard.putNumber("I Gain", self.kI)
        wpilib.SmartDashboard.putNumber("D Gain", self.kD)
        wpilib.SmartDashboard.putNumber("I Zone", self.kIz)
        wpilib.SmartDashboard.putNumber("Feed Forward", self.kFF)
        wpilib.SmartDashboard.putNumber("Min Output", self.kMinOutput)
        wpilib.SmartDashboard.putNumber("Max Output", self.kMaxOutput)
        wpilib.SmartDashboard.putNumber("Set Rotations", 0)

    def teleopPeriodic(self):
        # Read data from SmartDashboard
        p = wpilib.SmartDashboard.getNumber("P Gain", 0)
        i = wpilib.SmartDashboard.getNumber("I Gain", 0)
        d = wpilib.SmartDashboard.getNumber("D Gain", 0)
        iz = wpilib.SmartDashboard.getNumber("I Zone", 0)
        ff = wpilib.SmartDashboard.getNumber("Feed Forward", 0)
        min_out = wpilib.SmartDashboard.getNumber("Min Output", 0)
        max_out = wpilib.SmartDashboard.getNumber("Max Output", 0)
        rotations = wpilib.SmartDashboard.getNumber("Set Rotations", 0)

        # Update PIDController datapoints with the latest from SmartDashboard
        if p != self.kP:
            self.pidController.setP(p)
            self.kP = p
        if i != self.kI:
            self.pidController.setI(i)
            self.kI = i
        if d != self.kD:
            self.pidController.setD(d)
            self.kD = d
        if iz != self.kIz:
            self.pidController.setIZone(iz)
            self.kIz = iz
        if ff != self.kFF:
            self.pidController.setFF(ff)
            self.kFF = ff
        if (min_out != self.kMinOutput) or (max_out != self.kMaxOutput):
            self.pidController.setOutputRange(min_out, max_out)
            self.kMinOutput = min_out
            self.kMaxOutput = max_out

        # PIDController objects are commanded to a set point using the
        # setReference() method.
        #
        # The first parameter is the value of the set point, whose units vary
        # depending on the control type set in the second parameter.
        #
        # The second parameter is the control type can be set to one of four
        # parameters:
        # rev.ControlType.kDutyCycle
        # rev.ControlType.kPosition
        # rev.ControlType.kVelocity
        # rev.ControlType.kVoltage
        #
        # For more information on what these types are, refer to the Spark Max
        # documentation.
        self.pidController.setReference(rotations, rev.ControlType.kPosition)

        # Push Setpoint and the motor's current position to SmartDashboard.
        wpilib.SmartDashboard.putNumber("SetPoint", rotations)
        wpilib.SmartDashboard.putNumber("ProcessVariable", self.encoder.getPosition())


if __name__ == "__main__":
    wpilib.run(Robot)



# ----------------------------------------------------------------------------
# Copyright (c) 2017-2018 FIRST. All Rights Reserved.
# Open Source Software - may be modified and shared by FRC teams. The code
# must be accompanied by the FIRST BSD license file in the root directory of
# the project.
# ----------------------------------------------------------------------------

import rev
import wpilib


class Robot(wpilib.TimedRobot):
    def robotInit(self):
        # Create motor
        self.motor = rev.CANSparkMax(1, rev.MotorType.kBrushless)

        self.joystick = wpilib.Joystick(0)

        # The restoreFactoryDefaults method can be used to reset the
        # configuration parameters in the SPARK MAX to their factory default
        # state. If no argument is passed, these parameters will not persist
        # between power cycles
        self.motor.restoreFactoryDefaults()

        # Parameters can be set by calling the appropriate set() method on the
        # CANSparkMax object whose properties you want to change
        #
        # Set methods will return one of three CANError values which will let
        # you know if the parameter was successfully set:
        #   CANError.kOk
        #   CANError.kError
        #   CANError.kTimeout
        if self.motor.setIdleMode(rev.IdleMode.kCoast) is not rev.CANError.kOk:
            wpilib.SmartDashboard.putString("Idle Mode", "Error")

        # Similarly, parameters will have a get() method which allows you to
        # retrieve their values from the controller
        if self.motor.getIdleMode() is rev.IdleMode.kCoast:
            wpilib.SmartDashboard.putString("Idle Mode", "Coast")
        else:
            wpilib.SmartDashboard.putString("Idle Mode", "Brake")

        # Set ramp rate to 0
        if self.motor.setOpenLoopRampRate(0) is not rev.CANError.kOk:
            wpilib.SmartDashboard.putString("Ramp Rate", "Error")

        # Read back ramp value
        wpilib.SmartDashboard.putString(
            "Ramp Rate", str(self.motor.getOpenLoopRampRate())
        )

    def teleopPeriodic(self):
        # Pair motor and the joystick's Y Axis
        self.motor.set(self.joystick.getY())

        # Put Voltage, Temperature, and Motor Output onto SmartDashboard
        wpilib.SmartDashboard.putNumber("Voltage", self.motor.getBusVoltage())
        wpilib.SmartDashboard.putNumber("Temperature", self.motor.getMotorTemperature())
        wpilib.SmartDashboard.putNumber("Output", self.motor.getAppliedOutput())


if __name__ == "__main__":
    wpilib.run(Robot)



#
# This file generates the sidebar/toctree for all RobotPy projects and should
# be copied to each project when it is updated
#

import os


def write_if_changed(fname, contents):

    try:
        with open(fname, "r") as fp:
            old_contents = fp.read()
    except:
        old_contents = ""

    if old_contents != contents:
        with open(fname, "w") as fp:
            fp.write(contents)


def generate_sidebar(conf, conf_api):

    # determine 'latest' or 'stable'
    # if not conf.do_gen:
    do_gen = os.environ.get("SIDEBAR", None) == "1" or conf["on_rtd"]
    version = conf["rtd_version"]

    lines = ["", ".. DO NOT MODIFY! THIS PAGE IS AUTOGENERATED!", ""]

    def toctree(name):
        lines.extend(
            [".. toctree::", "    :caption: %s" % name, "    :maxdepth: 2", ""]
        )

    def endl():
        lines.append("")

    def write(desc, link):
        if conf_api == "robotpy":
            args = desc, link
        elif not do_gen:
            return
        else:
            args = (
                desc,
                "https://robotpy.readthedocs.io/en/%s/%s.html" % (version, link),
            )

        lines.append("    %s <%s>" % args)

    def write_api(project, desc):
        if project != conf_api:
            if do_gen:
                args = desc, project, version
                lines.append(
                    "    %s API <https://robotpy.readthedocs.io/projects/%s/en/%s/api.html>"
                    % args
                )
        else:
            lines.append("    %s API <api>" % desc)

    #
    # Specify the sidebar contents here
    #

    toctree("Robot Programming")
    write("Getting Started", "getting_started")
    write("Installation", "install/index")
    write("Programmer's Guide", "guide/index")
    write("Frameworks", "frameworks/index")
    write("Hardware & Sensors", "hw")
    write("Camera & Vision", "vision/index")
    endl()

    toctree("API Reference")
    write_api("wpilib", "WPILib")
    write_api("pynetworktables", "NetworkTables")
    write_api("cscore", "CSCore")
    write_api("utilities", "Utilities")
    write_api("pyfrc", "PyFRC")
    write_api("ctre", "CTRE Libraries")
    write_api("navx", "NavX Library")
    write_api("rev", "SPARK MAX Library")
    endl()

    toctree("Additional Info")
    write("Troubleshooting", "troubleshooting")
    write("Support", "support")
    write("FAQ", "faq")
    endl()

    toctree("RobotPy Developers")
    write("Developer Documentation", "dev/index")
    endl()

    write_if_changed("_sidebar.rst.inc", "\n".join(lines))



# ----------------------------------------------------------------------------
# Copyright (c) 2017-2018 FIRST. All Rights Reserved.
# Open Source Software - may be modified and shared by FRC teams. The code
# must be accompanied by the FIRST BSD license file in the root directory of
# the project.
# ----------------------------------------------------------------------------

import rev
import wpilib

# Before Running:
# Open Shuffleboard, select File->Load Layout and select the
# shuffleboard.json that is in the root directory of this example


class Robot(wpilib.TimedRobot):
    def robotInit(self):
        # Create motor
        self.motor = rev.CANSparkMax(1, rev.MotorType.kBrushless)

        self.joystick = wpilib.Joystick(0)

        # A CANDigitalInput object is constructed using the
        # GetForwardLimitSwitch() or
        # GetReverseLimitSwitch() method on an existing CANSparkMax object,
        # depending on which direction you would like to limit
        #
        # Limit switches can be configured to one of two polarities:
        # rev.CANDigitalInput.LimitSwitchPolarity.kNormallyOpen
        # rev.CANDigitalInput.LimitSwitchPolarity.kNormallyClosed
        self.forwardLimit = self.motor.getForwardLimitSwitch(
            rev.LimitSwitchPolarity.kNormallyClosed
        )
        self.reverseLimit = self.motor.getReverseLimitSwitch(
            rev.LimitSwitchPolarity.kNormallyClosed
        )

        self.forwardLimit.enableLimitSwitch(False)
        self.reverseLimit.enableLimitSwitch(False)

        wpilib.SmartDashboard.putBoolean(
            "Forward Limit Enabled", self.forwardLimit.isLimitSwitchEnabled()
        )
        wpilib.SmartDashboard.putBoolean(
            "Reverse Limit Enabled", self.forwardLimit.isLimitSwitchEnabled()
        )

    def teleopPeriodic(self):
        # Pair motor and the joystick's Y Axis
        self.motor.set(self.joystick.getY())

        # enable/disable limit switches based on value read from SmartDashboard
        self.forwardLimit.enableLimitSwitch(
            wpilib.SmartDashboard.getBoolean("Forward Limit Enabled", False)
        )
        self.reverseLimit.enableLimitSwitch(
            wpilib.SmartDashboard.getBoolean("Reverse Limit Enabled", False)
        )

        # The get() method can be used on a CANDigitalInput object to read the
        # state of the switch.
        #
        # In this example, the polarity of the switches are set to normally
        # closed. In this case, get() will return true if the switch is
        # pressed. It will also return true if you do not have a switch
        # connected. get() will return false when the switch is released.
        wpilib.SmartDashboard.putBoolean(
            "Forward Limit Switch", self.forwardLimit.get()
        )
        wpilib.SmartDashboard.putBoolean(
            "Reverse Limit Switch", self.reverseLimit.get()
        )


if __name__ == "__main__":
    wpilib.run(Robot)



# ----------------------------------------------------------------------------
# Copyright (c) 2017-2018 FIRST. All Rights Reserved.
# Open Source Software - may be modified and shared by FRC teams. The code
# must be accompanied by the FIRST BSD license file in the root directory of
# the project.
# ----------------------------------------------------------------------------

import rev
import wpilib
from wpilib.drive import DifferentialDrive


class Robot(wpilib.TimedRobot):
    def robotInit(self):
        # SPARK MAX controllers are intialized over CAN by constructing a
        # CANSparkMax object
        #
        # The CAN ID, which can be configured using the SPARK MAX Client, is passed
        # as the first parameter
        #
        # The motor type is passed as the second parameter.
        # Motor type can either be:
        #   rev.MotorType.kBrushless
        #   rev.MotorType.kBrushed
        #
        # The example below initializes four brushless motors with CAN IDs
        # 1, 2, 3, 4. Change these parameters to match your setup
        self.leftLeadMotor = rev.CANSparkMax(1, rev.MotorType.kBrushless)
        self.rightLeadMotor = rev.CANSparkMax(3, rev.MotorType.kBrushless)
        self.leftFollowMotor = rev.CANSparkMax(2, rev.MotorType.kBrushless)
        self.rightFollowMotor = rev.CANSparkMax(4, rev.MotorType.kBrushless)

        # Passing in the lead motors into DifferentialDrive allows any
        # commmands sent to the lead motors to be sent to the follower motors.
        self.driveTrain = DifferentialDrive(self.leftLeadMotor, self.rightLeadMotor)
        self.joystick = wpilib.Joystick(0)

        # The RestoreFactoryDefaults method can be used to reset the
        # configuration parameters in the SPARK MAX to their factory default
        # state. If no argument is passed, these parameters will not persist
        # between power cycles
        self.leftLeadMotor.restoreFactoryDefaults()
        self.rightLeadMotor.restoreFactoryDefaults()
        self.leftFollowMotor.restoreFactoryDefaults()
        self.rightFollowMotor.restoreFactoryDefaults()

        # In CAN mode, one SPARK MAX can be configured to follow another. This
        # is done by calling the follow() method on the SPARK MAX you want to
        # configure as a follower, and by passing as a parameter the SPARK MAX
        # you want to configure as a leader.
        #
        # This is shown in the example below, where one motor on each side of
        # our drive train is configured to follow a lead motor.
        self.leftFollowMotor.follow(self.leftLeadMotor)
        self.rightFollowMotor.follow(self.rightLeadMotor)

    def teleopPeriodic(self):
        # Drive with arcade style
        self.driveTrain.arcadeDrive(-self.joystick.getY(), self.joystick.getX())


if __name__ == "__main__":
    wpilib.run(Robot)



# ----------------------------------------------------------------------------
# Copyright (c) 2017-2018 FIRST. All Rights Reserved.
# Open Source Software - may be modified and shared by FRC teams. The code
# must be accompanied by the FIRST BSD license file in the root directory of
# the project.
# ----------------------------------------------------------------------------

import rev
import wpilib
from wpilib.drive import DifferentialDrive


class Robot(wpilib.TimedRobot):
    def robotInit(self):
        # SPARK MAX controllers are intialized over CAN by constructing a
        # CANSparkMax object
        #
        # The CAN ID, which can be configured using the SPARK MAX Client, is passed
        # as the first parameter
        #
        # The motor type is passed as the second parameter.
        # Motor type can either be:
        #   rev.MotorType.kBrushless
        #   rev.MotorType.kBrushed
        #
        # The example below initializes two brushless motors with CAN IDs
        # 1 and 2. Change these parameters to match your setup
        self.leftMotor = rev.CANSparkMax(1, rev.MotorType.kBrushless)
        self.rightMotor = rev.CANSparkMax(2, rev.MotorType.kBrushless)

        # The RestoreFactoryDefaults method can be used to reset the
        # configuration parameters in the SPARK MAX to their factory default
        # state. If no argument is passed, these parameters will not persist
        # between power cycles
        self.leftMotor.restoreFactoryDefaults()
        self.rightMotor.restoreFactoryDefaults()

        self.driveTrain = DifferentialDrive(self.leftMotor, self.rightMotor)
        self.l_stick = wpilib.Joystick(0)
        self.r_stick = wpilib.Joystick(1)

    def teleopPeriodic(self):
        # Create tank drive
        self.driveTrain.tankDrive(self.l_stick.getY(), self.r_stick.getY())


if __name__ == "__main__":
    wpilib.run(Robot)



#!/usr/bin/env python3
# ----------------------------------------------------------------------------
# Copyright (c) 2017-2018 FIRST. All Rights Reserved.
# Open Source Software - may be modified and shared by FRC teams. The code
# must be accompanied by the FIRST BSD license file in the root directory of
# the project.
# ----------------------------------------------------------------------------

import rev
import wpilib


# Before Running:
# Open Shuffleboard, select File->Load Layout and select the
# shuffleboard.json that is in the root directory of this example

# REV Smart Motion Guide
#
# The SPARK MAX includes a new control mode, REV Smart Motion which is used to
# control the position of the motor, and includes a max velocity and max
# acceleration parameter to ensure the motor moves in a smooth and predictable
# way. This is done by generating a motion profile on the fly in SPARK MAX and
# controlling the velocity of the motor to follow this profile.
#
# Since REV Smart Motion uses the velocity to track a profile, there are only
# two steps required to configure this mode:
#    1) Tune a velocity PID loop for the mechanism
#    2) Configure the smart motion parameters
#
# Tuning the Velocity PID Loop
#
# The most important part of tuning any closed loop control such as the velocity
# PID, is to graph the inputs and outputs to understand exactly what is happening.
# For tuning the Velocity PID loop, at a minimum we recommend graphing:
#
#    1) The velocity of the mechanism (‘Process variable’)
#    2) The commanded velocity value (‘Setpoint’)
#    3) The applied output
#
# This example will use ShuffleBoard to graph the above parameters. Make sure to
# load the shuffleboard.json file in the root of this directory to get the full
# effect of the GUI layout.


class Robot(wpilib.TimedRobot):
    def robotInit(self):
        # initialize motor
        self.motor = rev.CANSparkMax(1, rev.MotorType.kBrushless)

        # The RestoreFactoryDefaults method can be used to reset the configuration parameters
        # in the SPARK MAX to their factory default state. If no argument is passed, these
        # parameters will not persist between power cycles
        self.motor.restoreFactoryDefaults()

        self.pid_controller = self.motor.getPIDController()
        self.encoder = self.motor.getEncoder()

        # PID coefficients
        self.kP = 5e-5
        self.kI = 1e-6
        self.kD = 0
        self.kIz = 0
        self.kFF = 0.000156
        self.kMaxOutput = 1
        self.kMinOutput = -1
        self.max_rpm = 5700

        # Smart Motion coefficients
        self.max_vel = 1500  # rpm
        self.max_acc = 500
        self.min_vel = 0
        self.allowed_err = 0

        # set PID coefficients
        self.pid_controller.setP(self.kP)
        self.pid_controller.setI(self.kI)
        self.pid_controller.setD(self.kD)
        self.pid_controller.setIZone(self.kIz)
        self.pid_controller.setFF(self.kFF)
        self.pid_controller.setOutputRange(self.kMinOutput, self.kMaxOutput)

        # Smart Motion coefficients are set on a CANPIDController object
        #
        # - setSmartMotionMaxVelocity() will limit the velocity in RPM of
        # the pid controller in Smart Motion mode
        # - setSmartMotionMinOutputVelocity() will put a lower bound in
        # RPM of the pid controller in Smart Motion mode
        # - setSmartMotionMaxAccel() will limit the acceleration in RPM^2
        # of the pid controller in Smart Motion mode
        # - setSmartMotionAllowedClosedLoopError() will set the max allowed
        # error for the pid controller in Smart Motion mode
        smart_motion_slot = 0
        self.pid_controller.setSmartMotionMaxVelocity(self.max_vel, smart_motion_slot)
        self.pid_controller.setSmartMotionMinOutputVelocity(
            self.min_vel, smart_motion_slot
        )
        self.pid_controller.setSmartMotionMaxAccel(self.max_acc, smart_motion_slot)
        self.pid_controller.setSmartMotionAllowedClosedLoopError(
            self.allowed_err, smart_motion_slot
        )

        # display PID coefficients on SmartDashboard
        wpilib.SmartDashboard.putNumber("P Gain", self.kP)
        wpilib.SmartDashboard.putNumber("I Gain", self.kI)
        wpilib.SmartDashboard.putNumber("D Gain", self.kD)
        wpilib.SmartDashboard.putNumber("I Zone", self.kIz)
        wpilib.SmartDashboard.putNumber("Feed Forward", self.kFF)
        wpilib.SmartDashboard.putNumber("Max Output", self.kMaxOutput)
        wpilib.SmartDashboard.putNumber("Min Output", self.kMinOutput)

        # display Smart Motion coefficients
        wpilib.SmartDashboard.putNumber("Max Velocity", self.max_vel)
        wpilib.SmartDashboard.putNumber("Min Velocity", self.min_vel)
        wpilib.SmartDashboard.putNumber("Max Acceleration", self.max_acc)
        wpilib.SmartDashboard.putNumber("Allowed Closed Loop Error", self.allowed_err)
        wpilib.SmartDashboard.putNumber("Set Position", 0)
        wpilib.SmartDashboard.putNumber("Set Velocity", 0)

        # button to toggle between velocity and smart motion modes
        wpilib.SmartDashboard.putBoolean("Mode", True)

    def teleopPeriodic(self):
        # read PID coefficients from SmartDashboard
        p = wpilib.SmartDashboard.getNumber("P Gain", 0)
        i = wpilib.SmartDashboard.getNumber("I Gain", 0)
        d = wpilib.SmartDashboard.getNumber("D Gain", 0)
        iz = wpilib.SmartDashboard.getNumber("I Zone", 0)
        ff = wpilib.SmartDashboard.getNumber("Feed Forward", 0)
        max_out = wpilib.SmartDashboard.getNumber("Max Output", 0)
        min_out = wpilib.SmartDashboard.getNumber("Min Output", 0)
        maxV = wpilib.SmartDashboard.getNumber("Max Velocity", 0)
        minV = wpilib.SmartDashboard.getNumber("Min Velocity", 0)
        maxA = wpilib.SmartDashboard.getNumber("Max Acceleration", 0)
        allE = wpilib.SmartDashboard.getNumber("Allowed Closed Loop Error", 0)

        # if PID coefficients on SmartDashboard have changed, write new values to controller
        if p != self.kP:
            self.pid_controller.setP(p)
            self.kP = p
        if i != self.kI:
            self.pid_controller.setI(i)
            self.kI = i
        if d != self.kD:
            self.pid_controller.setD(d)
            self.kD = d
        if iz != self.kIz:
            self.pid_controller.setIZone(iz)
            self.kIz = iz
        if ff != self.kFF:
            self.pid_controller.setFF(ff)
            self.kFF = ff
        if max_out != self.kMaxOutput or min_out != self.kMinOutput:
            self.pid_controller.setOutputRange(min_out, max_out)
            self.kMinOutput = min_out
            self.kMaxOutput = max_out

        if maxV != self.max_vel:
            self.pid_controller.setSmartMotionMaxVelocity(maxV, 0)
            self.max_vel = maxV
        if minV != self.min_vel:
            self.pid_controller.setSmartMotionMinOutputVelocity(minV, 0)
            self.min_vel = minV
        if maxA != self.max_acc:
            self.pid_controller.setSmartMotionMaxAccel(maxA, 0)
            self.max_acc = maxA
        if allE != self.allowed_err:
            self.pid_controller.setSmartMotionAllowedClosedLoopError(allE, 0)
            self.allowed_err = allE

        mode = wpilib.SmartDashboard.getBoolean("Mode", False)
        if mode:
            setpoint = wpilib.SmartDashboard.getNumber("Set Velocity", 0)
            self.pid_controller.setReference(setpoint, rev.ControlType.kVelocity)
            pv = self.encoder.getVelocity()
        else:
            setpoint = wpilib.SmartDashboard.getNumber("Set Position", 0)
            # As with other PID modes, Smart Motion is set by calling the
            # setReference method on an existing pid object and setting
            # the control type to kSmartMotion
            self.pid_controller.setReference(setpoint, rev.ControlType.kSmartMotion)
            pv = self.encoder.getPosition()

        wpilib.SmartDashboard.putNumber("SetPoint", setpoint)
        wpilib.SmartDashboard.putNumber("Process Variable", pv)
        wpilib.SmartDashboard.putNumber("Output", self.motor.getAppliedOutput())


if __name__ == "__main__":
    wpilib.run(Robot)



import rev


def test_rev():
    pass


# def test_init(rev):
#     rev.CANSparkMax(0, rev.MotorType.kBrushless)


# def test_get_proxies(rev):
#     sm = rev.CANSparkMax(0, rev.MotorType.kBrushless)
#     sm.getEncoder()
#     sm.getPIDController()
#     sm.getForwardLimitSwitch(rev.LimitSwitchPolarity.kNormallyOpen)


# def test_current_limit(rev, hal_data):
#     sm = rev.CANSparkMax(1, rev.MotorType.kBrushless)

#     sm.setSecondaryCurrentLimit(50)

#     assert hal_data["CAN"]["sparkmax-1"]["currentChop"] == 50.0
#     assert isinstance(hal_data["CAN"]["sparkmax-1"]["currentChop"], float)
#     assert hal_data["CAN"]["sparkmax-1"]["currentChopCycles"] == 0

#     sm.setSecondaryCurrentLimit(52.5, 5)

#     assert hal_data["CAN"]["sparkmax-1"]["currentChop"] == 52.5
#     assert hal_data["CAN"]["sparkmax-1"]["currentChopCycles"] == 5


# def test_faults(rev, hal_data):
#     sm = rev.CANSparkMax(1, rev.MotorType.kBrushed)
#     rev_sw = sm.getReverseLimitSwitch(rev.LimitSwitchPolarity.kNormallyOpen)
#     hal_data["CAN"]["sparkmax-1"]["faults"][rev.FaultID.kHardLimitRev] = False
#     assert not rev_sw.get()


# def test_frame_period(rev, hal_data):
#     sm = rev.CANSparkMax(2, rev.MotorType.kBrushed)
#     sm.setPeriodicFramePeriod(rev.PeriodicFrame.kStatus2, 20)
#     assert (
#         hal_data["CAN"]["sparkmax-2"]["frame_period"][rev.PeriodicFrame.kStatus2] == 20
#     )


# def test_pid_set(rev, hal_data):
#     sm = rev.CANSparkMax(0, rev.MotorType.kBrushless)
#     pid = sm.getPIDController()
#     pid.setOutputRange(-1, 1)
#     pid.setP(0.005)
#     pid.setReference(5, rev.ControlType.kPosition)
